"use strict";
const classes_1 = require("../../classes");
function callSpawn(spawn, command, args = [], options = {}) {
    var _a, _b;
    const process = spawn(command, args, options);
    let stdout = '';
    let stderr = '';
    let output = '';
    (_a = process.stdout) === null || _a === void 0 ? void 0 : _a.on('data', chunk => {
        stdout += chunk;
        output += chunk;
    });
    (_b = process.stderr) === null || _b === void 0 ? void 0 : _b.on('data', chunk => {
        stderr += chunk;
        output += chunk;
    });
    const mkinfo = (status, signal) => ({
        command,
        args,
        options,
        stdout,
        stderr,
        output,
        status,
        signal,
        process
    });
    const onerror = new Promise(resolve => {
        process.on('error', error => resolve(new classes_1.InternalError({
            command,
            args,
            options,
            process,
            error
        })));
    });
    const createPromise = (event) => new Promise((resolve, reject) => {
        void onerror.then(error => reject(error));
        process.on(event, (status, signal) => {
            const info = mkinfo(status, signal);
            if (status) {
                reject(new classes_1.TerminationError(info));
            }
            else {
                resolve(info);
            }
        });
    });
    const [onclose, onexit] = (() => {
        const { event } = options;
        if (event) {
            const promise = createPromise(event);
            return [promise, promise];
        }
        else {
            return [
                createPromise('close'),
                createPromise('exit')
            ];
        }
    })();
    function silentPromiseRejection(promise) {
        void promise.catch(() => null);
    }
    return {
        get onclose() {
            silentPromiseRejection(onexit);
            return onclose;
        },
        get onexit() {
            silentPromiseRejection(onclose);
            return onexit;
        },
        process
    };
}
module.exports = callSpawn;
